/*
 * Copyright (c) Cloud Software Group, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2) Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package com.xensource.xenapi;

import com.xensource.xenapi.Types.BadServerResponse;
import com.xensource.xenapi.Types.XenAPIException;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import org.apache.xmlrpc.XmlRpcException;

/**
 * Pool-wide information
 * First published in XenServer 4.0.
 *
 * @author Cloud Software Group, Inc.
 */
public class Pool extends XenAPIObject {

    /**
     * The XenAPI reference (OpaqueRef) to this object.
     */
    protected final String ref;

    /**
     * For internal use only.
     */
    Pool(String ref) {
       this.ref = ref;
    }

    /**
     * @return The XenAPI reference (OpaqueRef) to this object.
     */
    public String toWireString() {
       return this.ref;
    }

    /**
     * If obj is a Pool, compares XenAPI references for equality.
     */
    @Override
    public boolean equals(Object obj)
    {
        if (obj instanceof Pool)
        {
            Pool other = (Pool) obj;
            return other.ref.equals(this.ref);
        } else
        {
            return false;
        }
    }

    @Override
    public int hashCode()
    {
        return ref.hashCode();
    }

    /**
     * Represents all the fields in a Pool
     */
    public static class Record implements Types.Record {
        public String toString() {
            StringWriter writer = new StringWriter();
            PrintWriter print = new PrintWriter(writer);
            print.printf("%1$20s: %2$s\n", "uuid", this.uuid);
            print.printf("%1$20s: %2$s\n", "nameLabel", this.nameLabel);
            print.printf("%1$20s: %2$s\n", "nameDescription", this.nameDescription);
            print.printf("%1$20s: %2$s\n", "master", this.master);
            print.printf("%1$20s: %2$s\n", "defaultSR", this.defaultSR);
            print.printf("%1$20s: %2$s\n", "suspendImageSR", this.suspendImageSR);
            print.printf("%1$20s: %2$s\n", "crashDumpSR", this.crashDumpSR);
            print.printf("%1$20s: %2$s\n", "otherConfig", this.otherConfig);
            print.printf("%1$20s: %2$s\n", "haEnabled", this.haEnabled);
            print.printf("%1$20s: %2$s\n", "haConfiguration", this.haConfiguration);
            print.printf("%1$20s: %2$s\n", "haStatefiles", this.haStatefiles);
            print.printf("%1$20s: %2$s\n", "haHostFailuresToTolerate", this.haHostFailuresToTolerate);
            print.printf("%1$20s: %2$s\n", "haPlanExistsFor", this.haPlanExistsFor);
            print.printf("%1$20s: %2$s\n", "haAllowOvercommit", this.haAllowOvercommit);
            print.printf("%1$20s: %2$s\n", "haOvercommitted", this.haOvercommitted);
            print.printf("%1$20s: %2$s\n", "blobs", this.blobs);
            print.printf("%1$20s: %2$s\n", "tags", this.tags);
            print.printf("%1$20s: %2$s\n", "guiConfig", this.guiConfig);
            print.printf("%1$20s: %2$s\n", "healthCheckConfig", this.healthCheckConfig);
            print.printf("%1$20s: %2$s\n", "wlbUrl", this.wlbUrl);
            print.printf("%1$20s: %2$s\n", "wlbUsername", this.wlbUsername);
            print.printf("%1$20s: %2$s\n", "wlbEnabled", this.wlbEnabled);
            print.printf("%1$20s: %2$s\n", "wlbVerifyCert", this.wlbVerifyCert);
            print.printf("%1$20s: %2$s\n", "redoLogEnabled", this.redoLogEnabled);
            print.printf("%1$20s: %2$s\n", "redoLogVdi", this.redoLogVdi);
            print.printf("%1$20s: %2$s\n", "vswitchController", this.vswitchController);
            print.printf("%1$20s: %2$s\n", "restrictions", this.restrictions);
            print.printf("%1$20s: %2$s\n", "metadataVDIs", this.metadataVDIs);
            print.printf("%1$20s: %2$s\n", "haClusterStack", this.haClusterStack);
            print.printf("%1$20s: %2$s\n", "allowedOperations", this.allowedOperations);
            print.printf("%1$20s: %2$s\n", "currentOperations", this.currentOperations);
            print.printf("%1$20s: %2$s\n", "guestAgentConfig", this.guestAgentConfig);
            print.printf("%1$20s: %2$s\n", "cpuInfo", this.cpuInfo);
            print.printf("%1$20s: %2$s\n", "policyNoVendorDevice", this.policyNoVendorDevice);
            print.printf("%1$20s: %2$s\n", "livePatchingDisabled", this.livePatchingDisabled);
            print.printf("%1$20s: %2$s\n", "igmpSnoopingEnabled", this.igmpSnoopingEnabled);
            print.printf("%1$20s: %2$s\n", "uefiCertificates", this.uefiCertificates);
            print.printf("%1$20s: %2$s\n", "customUefiCertificates", this.customUefiCertificates);
            print.printf("%1$20s: %2$s\n", "isPsrPending", this.isPsrPending);
            print.printf("%1$20s: %2$s\n", "tlsVerificationEnabled", this.tlsVerificationEnabled);
            print.printf("%1$20s: %2$s\n", "repositories", this.repositories);
            print.printf("%1$20s: %2$s\n", "clientCertificateAuthEnabled", this.clientCertificateAuthEnabled);
            print.printf("%1$20s: %2$s\n", "clientCertificateAuthName", this.clientCertificateAuthName);
            print.printf("%1$20s: %2$s\n", "repositoryProxyUrl", this.repositoryProxyUrl);
            print.printf("%1$20s: %2$s\n", "repositoryProxyUsername", this.repositoryProxyUsername);
            print.printf("%1$20s: %2$s\n", "repositoryProxyPassword", this.repositoryProxyPassword);
            print.printf("%1$20s: %2$s\n", "migrationCompression", this.migrationCompression);
            print.printf("%1$20s: %2$s\n", "coordinatorBias", this.coordinatorBias);
            print.printf("%1$20s: %2$s\n", "telemetryUuid", this.telemetryUuid);
            print.printf("%1$20s: %2$s\n", "telemetryFrequency", this.telemetryFrequency);
            print.printf("%1$20s: %2$s\n", "telemetryNextCollection", this.telemetryNextCollection);
            print.printf("%1$20s: %2$s\n", "lastUpdateSync", this.lastUpdateSync);
            print.printf("%1$20s: %2$s\n", "updateSyncFrequency", this.updateSyncFrequency);
            print.printf("%1$20s: %2$s\n", "updateSyncDay", this.updateSyncDay);
            print.printf("%1$20s: %2$s\n", "updateSyncEnabled", this.updateSyncEnabled);
            return writer.toString();
        }

        /**
         * Convert a pool.Record to a Map
         */
        public Map<String,Object> toMap() {
            Map<String,Object> map = new HashMap<>();
            map.put("uuid", this.uuid == null ? "" : this.uuid);
            map.put("name_label", this.nameLabel == null ? "" : this.nameLabel);
            map.put("name_description", this.nameDescription == null ? "" : this.nameDescription);
            map.put("master", this.master == null ? new Host("OpaqueRef:NULL") : this.master);
            map.put("default_SR", this.defaultSR == null ? new SR("OpaqueRef:NULL") : this.defaultSR);
            map.put("suspend_image_SR", this.suspendImageSR == null ? new SR("OpaqueRef:NULL") : this.suspendImageSR);
            map.put("crash_dump_SR", this.crashDumpSR == null ? new SR("OpaqueRef:NULL") : this.crashDumpSR);
            map.put("other_config", this.otherConfig == null ? new HashMap<String, String>() : this.otherConfig);
            map.put("ha_enabled", this.haEnabled == null ? false : this.haEnabled);
            map.put("ha_configuration", this.haConfiguration == null ? new HashMap<String, String>() : this.haConfiguration);
            map.put("ha_statefiles", this.haStatefiles == null ? new LinkedHashSet<String>() : this.haStatefiles);
            map.put("ha_host_failures_to_tolerate", this.haHostFailuresToTolerate == null ? 0 : this.haHostFailuresToTolerate);
            map.put("ha_plan_exists_for", this.haPlanExistsFor == null ? 0 : this.haPlanExistsFor);
            map.put("ha_allow_overcommit", this.haAllowOvercommit == null ? false : this.haAllowOvercommit);
            map.put("ha_overcommitted", this.haOvercommitted == null ? false : this.haOvercommitted);
            map.put("blobs", this.blobs == null ? new HashMap<String, Blob>() : this.blobs);
            map.put("tags", this.tags == null ? new LinkedHashSet<String>() : this.tags);
            map.put("gui_config", this.guiConfig == null ? new HashMap<String, String>() : this.guiConfig);
            map.put("health_check_config", this.healthCheckConfig == null ? new HashMap<String, String>() : this.healthCheckConfig);
            map.put("wlb_url", this.wlbUrl == null ? "" : this.wlbUrl);
            map.put("wlb_username", this.wlbUsername == null ? "" : this.wlbUsername);
            map.put("wlb_enabled", this.wlbEnabled == null ? false : this.wlbEnabled);
            map.put("wlb_verify_cert", this.wlbVerifyCert == null ? false : this.wlbVerifyCert);
            map.put("redo_log_enabled", this.redoLogEnabled == null ? false : this.redoLogEnabled);
            map.put("redo_log_vdi", this.redoLogVdi == null ? new VDI("OpaqueRef:NULL") : this.redoLogVdi);
            map.put("vswitch_controller", this.vswitchController == null ? "" : this.vswitchController);
            map.put("restrictions", this.restrictions == null ? new HashMap<String, String>() : this.restrictions);
            map.put("metadata_VDIs", this.metadataVDIs == null ? new LinkedHashSet<VDI>() : this.metadataVDIs);
            map.put("ha_cluster_stack", this.haClusterStack == null ? "" : this.haClusterStack);
            map.put("allowed_operations", this.allowedOperations == null ? new LinkedHashSet<Types.PoolAllowedOperations>() : this.allowedOperations);
            map.put("current_operations", this.currentOperations == null ? new HashMap<String, Types.PoolAllowedOperations>() : this.currentOperations);
            map.put("guest_agent_config", this.guestAgentConfig == null ? new HashMap<String, String>() : this.guestAgentConfig);
            map.put("cpu_info", this.cpuInfo == null ? new HashMap<String, String>() : this.cpuInfo);
            map.put("policy_no_vendor_device", this.policyNoVendorDevice == null ? false : this.policyNoVendorDevice);
            map.put("live_patching_disabled", this.livePatchingDisabled == null ? false : this.livePatchingDisabled);
            map.put("igmp_snooping_enabled", this.igmpSnoopingEnabled == null ? false : this.igmpSnoopingEnabled);
            map.put("uefi_certificates", this.uefiCertificates == null ? "" : this.uefiCertificates);
            map.put("custom_uefi_certificates", this.customUefiCertificates == null ? "" : this.customUefiCertificates);
            map.put("is_psr_pending", this.isPsrPending == null ? false : this.isPsrPending);
            map.put("tls_verification_enabled", this.tlsVerificationEnabled == null ? false : this.tlsVerificationEnabled);
            map.put("repositories", this.repositories == null ? new LinkedHashSet<Repository>() : this.repositories);
            map.put("client_certificate_auth_enabled", this.clientCertificateAuthEnabled == null ? false : this.clientCertificateAuthEnabled);
            map.put("client_certificate_auth_name", this.clientCertificateAuthName == null ? "" : this.clientCertificateAuthName);
            map.put("repository_proxy_url", this.repositoryProxyUrl == null ? "" : this.repositoryProxyUrl);
            map.put("repository_proxy_username", this.repositoryProxyUsername == null ? "" : this.repositoryProxyUsername);
            map.put("repository_proxy_password", this.repositoryProxyPassword == null ? new Secret("OpaqueRef:NULL") : this.repositoryProxyPassword);
            map.put("migration_compression", this.migrationCompression == null ? false : this.migrationCompression);
            map.put("coordinator_bias", this.coordinatorBias == null ? false : this.coordinatorBias);
            map.put("telemetry_uuid", this.telemetryUuid == null ? new Secret("OpaqueRef:NULL") : this.telemetryUuid);
            map.put("telemetry_frequency", this.telemetryFrequency == null ? Types.TelemetryFrequency.UNRECOGNIZED : this.telemetryFrequency);
            map.put("telemetry_next_collection", this.telemetryNextCollection == null ? new Date(0) : this.telemetryNextCollection);
            map.put("last_update_sync", this.lastUpdateSync == null ? new Date(0) : this.lastUpdateSync);
            map.put("update_sync_frequency", this.updateSyncFrequency == null ? Types.UpdateSyncFrequency.UNRECOGNIZED : this.updateSyncFrequency);
            map.put("update_sync_day", this.updateSyncDay == null ? 0 : this.updateSyncDay);
            map.put("update_sync_enabled", this.updateSyncEnabled == null ? false : this.updateSyncEnabled);
            return map;
        }

        /**
         * Unique identifier/object reference
         */
        public String uuid;
        /**
         * Short name
         */
        public String nameLabel;
        /**
         * Description
         */
        public String nameDescription;
        /**
         * The host that is pool master
         */
        public Host master;
        /**
         * Default SR for VDIs
         */
        public SR defaultSR;
        /**
         * The SR in which VDIs for suspend images are created
         */
        public SR suspendImageSR;
        /**
         * The SR in which VDIs for crash dumps are created
         */
        public SR crashDumpSR;
        /**
         * additional configuration
         */
        public Map<String, String> otherConfig;
        /**
         * true if HA is enabled on the pool, false otherwise
         * First published in XenServer 5.0.
         */
        public Boolean haEnabled;
        /**
         * The current HA configuration
         * First published in XenServer 5.0.
         */
        public Map<String, String> haConfiguration;
        /**
         * HA statefile VDIs in use
         * First published in XenServer 5.0.
         */
        public Set<String> haStatefiles;
        /**
         * Number of host failures to tolerate before the Pool is declared to be overcommitted
         * First published in XenServer 5.0.
         */
        public Long haHostFailuresToTolerate;
        /**
         * Number of future host failures we have managed to find a plan for. Once this reaches zero any future host failures will cause the failure of protected VMs.
         * First published in XenServer 5.0.
         */
        public Long haPlanExistsFor;
        /**
         * If set to false then operations which would cause the Pool to become overcommitted will be blocked.
         * First published in XenServer 5.0.
         */
        public Boolean haAllowOvercommit;
        /**
         * True if the Pool is considered to be overcommitted i.e. if there exist insufficient physical resources to tolerate the configured number of host failures
         * First published in XenServer 5.0.
         */
        public Boolean haOvercommitted;
        /**
         * Binary blobs associated with this pool
         * First published in XenServer 5.0.
         */
        public Map<String, Blob> blobs;
        /**
         * user-specified tags for categorization purposes
         * First published in XenServer 5.0.
         */
        public Set<String> tags;
        /**
         * gui-specific configuration for pool
         * First published in XenServer 5.0.
         */
        public Map<String, String> guiConfig;
        /**
         * Configuration for the automatic health check feature
         * First published in XenServer 7.0.
         */
        public Map<String, String> healthCheckConfig;
        /**
         * Url for the configured workload balancing host
         * First published in XenServer 5.5.
         */
        public String wlbUrl;
        /**
         * Username for accessing the workload balancing host
         * First published in XenServer 5.5.
         */
        public String wlbUsername;
        /**
         * true if workload balancing is enabled on the pool, false otherwise
         * First published in XenServer 5.5.
         */
        public Boolean wlbEnabled;
        /**
         * true if communication with the WLB server should enforce TLS certificate verification.
         * First published in XenServer 5.5.
         */
        public Boolean wlbVerifyCert;
        /**
         * true a redo-log is to be used other than when HA is enabled, false otherwise
         * First published in XenServer 5.6.
         */
        public Boolean redoLogEnabled;
        /**
         * indicates the VDI to use for the redo-log other than when HA is enabled
         * First published in XenServer 5.6.
         */
        public VDI redoLogVdi;
        /**
         * address of the vswitch controller
         * First published in XenServer 5.6.
         */
        public String vswitchController;
        /**
         * Pool-wide restrictions currently in effect
         * First published in XenServer 5.6.
         */
        public Map<String, String> restrictions;
        /**
         * The set of currently known metadata VDIs for this pool
         * First published in XenServer 6.0.
         */
        public Set<VDI> metadataVDIs;
        /**
         * The HA cluster stack that is currently in use. Only valid when HA is enabled.
         * First published in XenServer 7.0.
         */
        public String haClusterStack;
        /**
         * list of the operations allowed in this state. This list is advisory only and the server state may have changed by the time this field is read by a client.
         */
        public Set<Types.PoolAllowedOperations> allowedOperations;
        /**
         * links each of the running tasks using this object (by reference) to a current_operation enum which describes the nature of the task.
         */
        public Map<String, Types.PoolAllowedOperations> currentOperations;
        /**
         * Pool-wide guest agent configuration information
         * First published in XenServer 7.0.
         */
        public Map<String, String> guestAgentConfig;
        /**
         * Details about the physical CPUs on the pool
         * First published in XenServer 7.0.
         */
        public Map<String, String> cpuInfo;
        /**
         * The pool-wide policy for clients on whether to use the vendor device or not on newly created VMs. This field will also be consulted if the 'has_vendor_device' field is not specified in the VM.create call.
         * First published in XenServer 7.0.
         */
        public Boolean policyNoVendorDevice;
        /**
         * The pool-wide flag to show if the live patching feauture is disabled or not.
         * First published in XenServer 7.1.
         */
        public Boolean livePatchingDisabled;
        /**
         * true if IGMP snooping is enabled in the pool, false otherwise.
         * First published in XenServer 7.3.
         */
        public Boolean igmpSnoopingEnabled;
        /**
         * The UEFI certificates allowing Secure Boot
         * First published in Citrix Hypervisor 8.1.
         */
        public String uefiCertificates;
        /**
         * Custom UEFI certificates allowing Secure Boot
         * Experimental. First published in 23.25.0-next.
         */
        public String customUefiCertificates;
        /**
         * True if either a PSR is running or we are waiting for a PSR to be re-run
         * First published in Citrix Hypervisor 8.2 Hotfix 2.
         */
        public Boolean isPsrPending;
        /**
         * True iff TLS certificate verification is enabled
         * First published in 1.290.0.
         */
        public Boolean tlsVerificationEnabled;
        /**
         * The set of currently enabled repositories
         * First published in 1.301.0.
         */
        public Set<Repository> repositories;
        /**
         * True if authentication by TLS client certificates is enabled
         * First published in 1.318.0.
         */
        public Boolean clientCertificateAuthEnabled;
        /**
         * The name (CN/SAN) that an incoming client certificate must have to allow authentication
         * First published in 1.318.0.
         */
        public String clientCertificateAuthName;
        /**
         * Url of the proxy used in syncing with the enabled repositories
         * First published in 21.3.0.
         */
        public String repositoryProxyUrl;
        /**
         * Username for the authentication of the proxy used in syncing with the enabled repositories
         * First published in 21.3.0.
         */
        public String repositoryProxyUsername;
        /**
         * Password for the authentication of the proxy used in syncing with the enabled repositories
         * First published in 21.3.0.
         */
        public Secret repositoryProxyPassword;
        /**
         * Default behaviour during migration, True if stream compression should be used
         * Experimental. First published in 22.33.0.
         */
        public Boolean migrationCompression;
        /**
         * true if bias against pool master when scheduling vms is enabled, false otherwise
         */
        public Boolean coordinatorBias;
        /**
         * The UUID of the pool for identification of telemetry data
         * Experimental. First published in 23.9.0.
         */
        public Secret telemetryUuid;
        /**
         * How often the telemetry collection will be carried out
         * Experimental. First published in 23.9.0.
         */
        public Types.TelemetryFrequency telemetryFrequency;
        /**
         * The earliest timestamp (in UTC) when the next round of telemetry collection can be carried out
         * Experimental. First published in 23.9.0.
         */
        public Date telemetryNextCollection;
        /**
         * time of the last update sychronization
         * Experimental. First published in 23.18.0.
         */
        public Date lastUpdateSync;
        /**
         * The frequency at which updates are synchronized from a remote CDN: daily or weekly.
         * Experimental. First published in 23.18.0.
         */
        public Types.UpdateSyncFrequency updateSyncFrequency;
        /**
         * The day of the week the update synchronizations will be scheduled, based on pool's local timezone. Ignored when update_sync_frequency is daily
         * Experimental. First published in 23.18.0.
         */
        public Long updateSyncDay;
        /**
         * Whether periodic update synchronization is enabled or not
         * Experimental. First published in 23.18.0.
         */
        public Boolean updateSyncEnabled;
    }

    /**
     * Get a record containing the current state of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return all fields from the object
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Pool.Record getRecord(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_record";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toPoolRecord(result);
    }

    /**
     * Get a reference to the pool instance with the specified UUID.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param uuid UUID of object to return
     * @return reference to the object
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Pool getByUuid(Connection c, String uuid) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_by_uuid";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(uuid)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toPool(result);
    }

    /**
     * Get the uuid field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getUuid(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_uuid";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the name_label field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getNameLabel(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_name_label";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the name_description field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getNameDescription(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_name_description";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the master field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Host getMaster(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_master";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toHost(result);
    }

    /**
     * Get the default_SR field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public SR getDefaultSR(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_default_SR";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSR(result);
    }

    /**
     * Get the suspend_image_SR field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public SR getSuspendImageSR(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_suspend_image_SR";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSR(result);
    }

    /**
     * Get the crash_dump_SR field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public SR getCrashDumpSR(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_crash_dump_SR";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSR(result);
    }

    /**
     * Get the other_config field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, String> getOtherConfig(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_other_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Get the ha_enabled field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getHaEnabled(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_ha_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the ha_configuration field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, String> getHaConfiguration(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_ha_configuration";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Get the ha_statefiles field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Set<String> getHaStatefiles(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_ha_statefiles";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfString(result);
    }

    /**
     * Get the ha_host_failures_to_tolerate field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Long getHaHostFailuresToTolerate(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_ha_host_failures_to_tolerate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toLong(result);
    }

    /**
     * Get the ha_plan_exists_for field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Long getHaPlanExistsFor(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_ha_plan_exists_for";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toLong(result);
    }

    /**
     * Get the ha_allow_overcommit field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getHaAllowOvercommit(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_ha_allow_overcommit";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the ha_overcommitted field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getHaOvercommitted(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_ha_overcommitted";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the blobs field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, Blob> getBlobs(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_blobs";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringBlob(result);
    }

    /**
     * Get the tags field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Set<String> getTags(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_tags";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfString(result);
    }

    /**
     * Get the gui_config field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, String> getGuiConfig(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_gui_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Get the health_check_config field of the given pool.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, String> getHealthCheckConfig(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_health_check_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Get the wlb_url field of the given pool.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getWlbUrl(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_wlb_url";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the wlb_username field of the given pool.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getWlbUsername(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_wlb_username";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the wlb_enabled field of the given pool.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getWlbEnabled(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_wlb_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the wlb_verify_cert field of the given pool.
     * First published in XenServer 5.5.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public Boolean getWlbVerifyCert(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_wlb_verify_cert";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the redo_log_enabled field of the given pool.
     * First published in XenServer 5.6.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getRedoLogEnabled(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_redo_log_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the redo_log_vdi field of the given pool.
     * First published in XenServer 5.6.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public VDI getRedoLogVdi(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_redo_log_vdi";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toVDI(result);
    }

    /**
     * Get the vswitch_controller field of the given pool.
     * First published in XenServer 5.6.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public String getVswitchController(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_vswitch_controller";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the restrictions field of the given pool.
     * First published in XenServer 5.6.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, String> getRestrictions(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_restrictions";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Get the metadata_VDIs field of the given pool.
     * First published in XenServer 6.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Set<VDI> getMetadataVDIs(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_metadata_VDIs";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfVDI(result);
    }

    /**
     * Get the ha_cluster_stack field of the given pool.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getHaClusterStack(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_ha_cluster_stack";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the allowed_operations field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Set<Types.PoolAllowedOperations> getAllowedOperations(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_allowed_operations";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfPoolAllowedOperations(result);
    }

    /**
     * Get the current_operations field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, Types.PoolAllowedOperations> getCurrentOperations(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_current_operations";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringPoolAllowedOperations(result);
    }

    /**
     * Get the guest_agent_config field of the given pool.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, String> getGuestAgentConfig(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_guest_agent_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Get the cpu_info field of the given pool.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, String> getCpuInfo(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_cpu_info";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Get the policy_no_vendor_device field of the given pool.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getPolicyNoVendorDevice(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_policy_no_vendor_device";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the live_patching_disabled field of the given pool.
     * First published in XenServer 7.1.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getLivePatchingDisabled(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_live_patching_disabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the igmp_snooping_enabled field of the given pool.
     * First published in XenServer 7.3.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getIgmpSnoopingEnabled(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_igmp_snooping_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the uefi_certificates field of the given pool.
     * First published in Citrix Hypervisor 8.1.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getUefiCertificates(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_uefi_certificates";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the custom_uefi_certificates field of the given pool.
     * Experimental. First published in 23.25.0-next.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getCustomUefiCertificates(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_custom_uefi_certificates";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the is_psr_pending field of the given pool.
     * First published in Citrix Hypervisor 8.2 Hotfix 2.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getIsPsrPending(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_is_psr_pending";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the tls_verification_enabled field of the given pool.
     * First published in 1.290.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getTlsVerificationEnabled(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_tls_verification_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the repositories field of the given pool.
     * First published in 1.301.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Set<Repository> getRepositories(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_repositories";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfRepository(result);
    }

    /**
     * Get the client_certificate_auth_enabled field of the given pool.
     * First published in 1.318.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getClientCertificateAuthEnabled(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_client_certificate_auth_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the client_certificate_auth_name field of the given pool.
     * First published in 1.318.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getClientCertificateAuthName(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_client_certificate_auth_name";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the repository_proxy_url field of the given pool.
     * First published in 21.3.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getRepositoryProxyUrl(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_repository_proxy_url";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the repository_proxy_username field of the given pool.
     * First published in 21.3.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String getRepositoryProxyUsername(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_repository_proxy_username";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Get the repository_proxy_password field of the given pool.
     * First published in 21.3.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Secret getRepositoryProxyPassword(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_repository_proxy_password";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSecret(result);
    }

    /**
     * Get the migration_compression field of the given pool.
     * Experimental. First published in 22.33.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getMigrationCompression(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_migration_compression";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the coordinator_bias field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getCoordinatorBias(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_coordinator_bias";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Get the telemetry_uuid field of the given pool.
     * Experimental. First published in 23.9.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Secret getTelemetryUuid(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_telemetry_uuid";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSecret(result);
    }

    /**
     * Get the telemetry_frequency field of the given pool.
     * Experimental. First published in 23.9.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Types.TelemetryFrequency getTelemetryFrequency(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_telemetry_frequency";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTelemetryFrequency(result);
    }

    /**
     * Get the telemetry_next_collection field of the given pool.
     * Experimental. First published in 23.9.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Date getTelemetryNextCollection(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_telemetry_next_collection";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toDate(result);
    }

    /**
     * Get the last_update_sync field of the given pool.
     * Experimental. First published in 23.18.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Date getLastUpdateSync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_last_update_sync";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toDate(result);
    }

    /**
     * Get the update_sync_frequency field of the given pool.
     * Experimental. First published in 23.18.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Types.UpdateSyncFrequency getUpdateSyncFrequency(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_update_sync_frequency";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toUpdateSyncFrequency(result);
    }

    /**
     * Get the update_sync_day field of the given pool.
     * Experimental. First published in 23.18.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Long getUpdateSyncDay(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_update_sync_day";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toLong(result);
    }

    /**
     * Get the update_sync_enabled field of the given pool.
     * Experimental. First published in 23.18.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean getUpdateSyncEnabled(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_update_sync_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Set the name_label field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param nameLabel New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setNameLabel(Connection c, String nameLabel) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_name_label";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(nameLabel)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the name_description field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param nameDescription New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setNameDescription(Connection c, String nameDescription) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_name_description";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(nameDescription)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the default_SR field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param defaultSR New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setDefaultSR(Connection c, SR defaultSR) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_default_SR";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(defaultSR)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the suspend_image_SR field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param suspendImageSR New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setSuspendImageSR(Connection c, SR suspendImageSR) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_suspend_image_SR";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(suspendImageSR)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the crash_dump_SR field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param crashDumpSR New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setCrashDumpSR(Connection c, SR crashDumpSR) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_crash_dump_SR";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(crashDumpSR)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the other_config field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param otherConfig New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setOtherConfig(Connection c, Map<String, String> otherConfig) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_other_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(otherConfig)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Add the given key-value pair to the other_config field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param key Key to add
     * @param value Value to add
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void addToOtherConfig(Connection c, String key, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.add_to_other_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove the given key and its corresponding value from the other_config field of the given pool.  If the key is not in that Map, then do nothing.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param key Key to remove
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void removeFromOtherConfig(Connection c, String key) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.remove_from_other_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the ha_allow_overcommit field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param haAllowOvercommit New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setHaAllowOvercommit(Connection c, Boolean haAllowOvercommit) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_ha_allow_overcommit";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(haAllowOvercommit)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the tags field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param tags New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setTags(Connection c, Set<String> tags) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_tags";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(tags)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Add the given value to the tags field of the given pool.  If the value is already in that Set, then do nothing.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param value New value to add
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void addTags(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.add_tags";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove the given value from the tags field of the given pool.  If the value is not in that Set, then do nothing.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param value Value to remove
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void removeTags(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.remove_tags";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the gui_config field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param guiConfig New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setGuiConfig(Connection c, Map<String, String> guiConfig) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_gui_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(guiConfig)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Add the given key-value pair to the gui_config field of the given pool.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param key Key to add
     * @param value Value to add
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void addToGuiConfig(Connection c, String key, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.add_to_gui_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove the given key and its corresponding value from the gui_config field of the given pool.  If the key is not in that Map, then do nothing.
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param key Key to remove
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void removeFromGuiConfig(Connection c, String key) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.remove_from_gui_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the health_check_config field of the given pool.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param healthCheckConfig New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setHealthCheckConfig(Connection c, Map<String, String> healthCheckConfig) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_health_check_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(healthCheckConfig)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Add the given key-value pair to the health_check_config field of the given pool.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param key Key to add
     * @param value Value to add
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void addToHealthCheckConfig(Connection c, String key, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.add_to_health_check_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove the given key and its corresponding value from the health_check_config field of the given pool.  If the key is not in that Map, then do nothing.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param key Key to remove
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void removeFromHealthCheckConfig(Connection c, String key) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.remove_from_health_check_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the wlb_enabled field of the given pool.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param wlbEnabled New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setWlbEnabled(Connection c, Boolean wlbEnabled) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_wlb_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(wlbEnabled)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the wlb_verify_cert field of the given pool.
     * First published in XenServer 5.5.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param wlbVerifyCert New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public void setWlbVerifyCert(Connection c, Boolean wlbVerifyCert) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_wlb_verify_cert";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(wlbVerifyCert)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the policy_no_vendor_device field of the given pool.
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param policyNoVendorDevice New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setPolicyNoVendorDevice(Connection c, Boolean policyNoVendorDevice) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_policy_no_vendor_device";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(policyNoVendorDevice)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the live_patching_disabled field of the given pool.
     * First published in XenServer 7.1.
     *
     * @param c The connection the call is made on
     * @param livePatchingDisabled New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setLivePatchingDisabled(Connection c, Boolean livePatchingDisabled) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_live_patching_disabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(livePatchingDisabled)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the is_psr_pending field of the given pool.
     * First published in Citrix Hypervisor 8.2 Hotfix 2.
     *
     * @param c The connection the call is made on
     * @param isPsrPending New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setIsPsrPending(Connection c, Boolean isPsrPending) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_is_psr_pending";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(isPsrPending)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the migration_compression field of the given pool.
     * Experimental. First published in 22.33.0.
     *
     * @param c The connection the call is made on
     * @param migrationCompression New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setMigrationCompression(Connection c, Boolean migrationCompression) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_migration_compression";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(migrationCompression)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the coordinator_bias field of the given pool.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param coordinatorBias New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setCoordinatorBias(Connection c, Boolean coordinatorBias) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_coordinator_bias";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(coordinatorBias)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Instruct host to join a new pool
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param masterAddress The hostname of the master of the pool to join
     * @param masterUsername The username of the master (for initial authentication)
     * @param masterPassword The password for the master (for initial authentication)
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.JoiningHostCannotContainSharedSrs The server joining the pool cannot contain any shared storage.
     */
    public static Task joinAsync(Connection c, String masterAddress, String masterUsername, String masterPassword) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.JoiningHostCannotContainSharedSrs {
        String method_call = "Async.pool.join";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(masterAddress), Marshalling.toXMLRPC(masterUsername), Marshalling.toXMLRPC(masterPassword)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Instruct host to join a new pool
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param masterAddress The hostname of the master of the pool to join
     * @param masterUsername The username of the master (for initial authentication)
     * @param masterPassword The password for the master (for initial authentication)
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.JoiningHostCannotContainSharedSrs The server joining the pool cannot contain any shared storage.
     */
    public static void join(Connection c, String masterAddress, String masterUsername, String masterPassword) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.JoiningHostCannotContainSharedSrs {
        String method_call = "pool.join";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(masterAddress), Marshalling.toXMLRPC(masterUsername), Marshalling.toXMLRPC(masterPassword)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Instruct host to join a new pool
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param masterAddress The hostname of the master of the pool to join
     * @param masterUsername The username of the master (for initial authentication)
     * @param masterPassword The password for the master (for initial authentication)
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task joinForceAsync(Connection c, String masterAddress, String masterUsername, String masterPassword) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.join_force";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(masterAddress), Marshalling.toXMLRPC(masterUsername), Marshalling.toXMLRPC(masterPassword)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Instruct host to join a new pool
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param masterAddress The hostname of the master of the pool to join
     * @param masterUsername The username of the master (for initial authentication)
     * @param masterPassword The password for the master (for initial authentication)
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void joinForce(Connection c, String masterAddress, String masterUsername, String masterPassword) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.join_force";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(masterAddress), Marshalling.toXMLRPC(masterUsername), Marshalling.toXMLRPC(masterPassword)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Instruct a pool master to eject a host from the pool
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param host The host to eject
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task ejectAsync(Connection c, Host host) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.eject";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(host)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Instruct a pool master to eject a host from the pool
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param host The host to eject
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void eject(Connection c, Host host) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.eject";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(host)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Instruct host that's currently a slave to transition to being master
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void emergencyTransitionToMaster(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.emergency_transition_to_master";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Instruct a slave already in a pool that the master has changed
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param masterAddress The hostname of the master
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void emergencyResetMaster(Connection c, String masterAddress) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.emergency_reset_master";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(masterAddress)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Instruct a pool master, M, to try and contact its slaves and, if slaves are in emergency mode, reset their master address to M.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task recoverSlavesAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.recover_slaves";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Instruct a pool master, M, to try and contact its slaves and, if slaves are in emergency mode, reset their master address to M.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return list of hosts whose master address were successfully reset
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Set<Host> recoverSlaves(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.recover_slaves";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfHost(result);
    }

    /**
     * Create PIFs, mapping a network to the same physical interface/VLAN on each host. This call is deprecated: use Pool.create_VLAN_from_PIF instead.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param device physical interface on which to create the VLAN interface
     * @param network network to which this interface should be connected
     * @param VLAN VLAN tag for the new interface
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.VlanTagInvalid You tried to create a VLAN, but the tag you gave was invalid -- it must be between 0 and 4094. The parameter echoes the VLAN tag you gave.
     */
    public static Task createVLANAsync(Connection c, String device, Network network, Long VLAN) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.VlanTagInvalid {
        String method_call = "Async.pool.create_VLAN";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(device), Marshalling.toXMLRPC(network), Marshalling.toXMLRPC(VLAN)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Create PIFs, mapping a network to the same physical interface/VLAN on each host. This call is deprecated: use Pool.create_VLAN_from_PIF instead.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param device physical interface on which to create the VLAN interface
     * @param network network to which this interface should be connected
     * @param VLAN VLAN tag for the new interface
     * @return The references of the created PIF objects
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.VlanTagInvalid You tried to create a VLAN, but the tag you gave was invalid -- it must be between 0 and 4094. The parameter echoes the VLAN tag you gave.
     */
    public static Set<PIF> createVLAN(Connection c, String device, Network network, Long VLAN) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.VlanTagInvalid {
        String method_call = "pool.create_VLAN";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(device), Marshalling.toXMLRPC(network), Marshalling.toXMLRPC(VLAN)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfPIF(result);
    }

    /**
     * Reconfigure the management network interface for all Hosts in the Pool
     * First published in XenServer 7.3.
     *
     * @param c The connection the call is made on
     * @param network The network
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.HaIsEnabled The operation could not be performed because HA is enabled on the Pool
     * @throws Types.PifNotPresent This host has no PIF on the given network.
     * @throws Types.CannotPlugBondSlave This PIF is a bond member and cannot be plugged.
     * @throws Types.PifIncompatiblePrimaryAddressType The primary address types are not compatible
     * @throws Types.PifHasNoNetworkConfiguration PIF has no IP configuration (mode currently set to 'none')
     * @throws Types.PifHasNoV6NetworkConfiguration PIF has no IPv6 configuration (mode currently set to 'none')
     */
    public static Task managementReconfigureAsync(Connection c, Network network) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.HaIsEnabled,
       Types.PifNotPresent,
       Types.CannotPlugBondSlave,
       Types.PifIncompatiblePrimaryAddressType,
       Types.PifHasNoNetworkConfiguration,
       Types.PifHasNoV6NetworkConfiguration {
        String method_call = "Async.pool.management_reconfigure";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(network)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Reconfigure the management network interface for all Hosts in the Pool
     * First published in XenServer 7.3.
     *
     * @param c The connection the call is made on
     * @param network The network
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.HaIsEnabled The operation could not be performed because HA is enabled on the Pool
     * @throws Types.PifNotPresent This host has no PIF on the given network.
     * @throws Types.CannotPlugBondSlave This PIF is a bond member and cannot be plugged.
     * @throws Types.PifIncompatiblePrimaryAddressType The primary address types are not compatible
     * @throws Types.PifHasNoNetworkConfiguration PIF has no IP configuration (mode currently set to 'none')
     * @throws Types.PifHasNoV6NetworkConfiguration PIF has no IPv6 configuration (mode currently set to 'none')
     */
    public static void managementReconfigure(Connection c, Network network) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.HaIsEnabled,
       Types.PifNotPresent,
       Types.CannotPlugBondSlave,
       Types.PifIncompatiblePrimaryAddressType,
       Types.PifHasNoNetworkConfiguration,
       Types.PifHasNoV6NetworkConfiguration {
        String method_call = "pool.management_reconfigure";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(network)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Create a pool-wide VLAN by taking the PIF.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param pif physical interface on any particular host, that identifies the PIF on which to create the (pool-wide) VLAN interface
     * @param network network to which this interface should be connected
     * @param VLAN VLAN tag for the new interface
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.VlanTagInvalid You tried to create a VLAN, but the tag you gave was invalid -- it must be between 0 and 4094. The parameter echoes the VLAN tag you gave.
     */
    public static Task createVLANFromPIFAsync(Connection c, PIF pif, Network network, Long VLAN) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.VlanTagInvalid {
        String method_call = "Async.pool.create_VLAN_from_PIF";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(pif), Marshalling.toXMLRPC(network), Marshalling.toXMLRPC(VLAN)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Create a pool-wide VLAN by taking the PIF.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param pif physical interface on any particular host, that identifies the PIF on which to create the (pool-wide) VLAN interface
     * @param network network to which this interface should be connected
     * @param VLAN VLAN tag for the new interface
     * @return The references of the created PIF objects
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.VlanTagInvalid You tried to create a VLAN, but the tag you gave was invalid -- it must be between 0 and 4094. The parameter echoes the VLAN tag you gave.
     */
    public static Set<PIF> createVLANFromPIF(Connection c, PIF pif, Network network, Long VLAN) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.VlanTagInvalid {
        String method_call = "pool.create_VLAN_from_PIF";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(pif), Marshalling.toXMLRPC(network), Marshalling.toXMLRPC(VLAN)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfPIF(result);
    }

    /**
     * Turn on High Availability mode
     * First published in XenServer 4.1.
     *
     * @param c The connection the call is made on
     * @param heartbeatSrs Set of SRs to use for storage heartbeating
     * @param configuration Detailed HA configuration to apply
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task enableHaAsync(Connection c, Set<SR> heartbeatSrs, Map<String, String> configuration) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.enable_ha";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(heartbeatSrs), Marshalling.toXMLRPC(configuration)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Turn on High Availability mode
     * First published in XenServer 4.1.
     *
     * @param c The connection the call is made on
     * @param heartbeatSrs Set of SRs to use for storage heartbeating
     * @param configuration Detailed HA configuration to apply
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void enableHa(Connection c, Set<SR> heartbeatSrs, Map<String, String> configuration) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.enable_ha";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(heartbeatSrs), Marshalling.toXMLRPC(configuration)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Turn off High Availability mode
     * First published in XenServer 4.1.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task disableHaAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.disable_ha";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Turn off High Availability mode
     * First published in XenServer 4.1.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void disableHa(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.disable_ha";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Forcibly synchronise the database now
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task syncDatabaseAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.sync_database";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Forcibly synchronise the database now
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void syncDatabase(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.sync_database";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Perform an orderly handover of the role of master to the referenced host.
     * First published in XenServer 4.1.
     *
     * @param c The connection the call is made on
     * @param host The host who should become the new master
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task designateNewMasterAsync(Connection c, Host host) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.designate_new_master";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(host)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Perform an orderly handover of the role of master to the referenced host.
     * First published in XenServer 4.1.
     *
     * @param c The connection the call is made on
     * @param host The host who should become the new master
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void designateNewMaster(Connection c, Host host) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.designate_new_master";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(host)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * When this call returns the VM restart logic will not run for the requested number of seconds. If the argument is zero then the restart thread is immediately unblocked
     * First published in XenServer 5.0 Update 1.
     *
     * @param c The connection the call is made on
     * @param seconds The number of seconds to block the restart thread for
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void haPreventRestartsFor(Connection c, Long seconds) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.ha_prevent_restarts_for";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(seconds)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Returns true if a VM failover plan exists for up to 'n' host failures
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param n The number of host failures to plan for
     * @return true if a failover plan exists for the supplied number of host failures
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Boolean haFailoverPlanExists(Connection c, Long n) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.ha_failover_plan_exists";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(n)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Returns the maximum number of host failures we could tolerate before we would be unable to restart configured VMs
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @return maximum value for ha_host_failures_to_tolerate given current configuration
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Long haComputeMaxHostFailuresToTolerate(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.ha_compute_max_host_failures_to_tolerate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toLong(result);
    }

    /**
     * Returns the maximum number of host failures we could tolerate before we would be unable to restart the provided VMs
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param configuration Map of protected VM reference to restart priority
     * @return maximum value for ha_host_failures_to_tolerate given provided configuration
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Long haComputeHypotheticalMaxHostFailuresToTolerate(Connection c, Map<VM, String> configuration) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.ha_compute_hypothetical_max_host_failures_to_tolerate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(configuration)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toLong(result);
    }

    /**
     * Return a VM failover plan assuming a given subset of hosts fail
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param failedHosts The set of hosts to assume have failed
     * @param failedVms The set of VMs to restart
     * @return VM failover plan: a map of VM to host to restart the host on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Map<VM, Map<String, String>> haComputeVmFailoverPlan(Connection c, Set<Host> failedHosts, Set<VM> failedVms) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.ha_compute_vm_failover_plan";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(failedHosts), Marshalling.toXMLRPC(failedVms)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfVMMapOfStringString(result);
    }

    /**
     * Set the maximum number of host failures to consider in the HA VM restart planner
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param value New number of host failures to consider
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task setHaHostFailuresToTolerateAsync(Connection c, Long value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_ha_host_failures_to_tolerate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Set the maximum number of host failures to consider in the HA VM restart planner
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param value New number of host failures to consider
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setHaHostFailuresToTolerate(Connection c, Long value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_ha_host_failures_to_tolerate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Create a placeholder for a named binary blob of data that is associated with this pool
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param name The name associated with the blob
     * @param mimeType The mime type for the data. Empty string translates to application/octet-stream
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task createNewBlobAsync(Connection c, String name, String mimeType) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.create_new_blob";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(mimeType)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Create a placeholder for a named binary blob of data that is associated with this pool
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param name The name associated with the blob
     * @param mimeType The mime type for the data. Empty string translates to application/octet-stream
     * @return The reference of the blob, needed for populating its data
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Blob createNewBlob(Connection c, String name, String mimeType) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.create_new_blob";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(mimeType)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBlob(result);
    }

    /**
     * Create a placeholder for a named binary blob of data that is associated with this pool
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param name The name associated with the blob
     * @param mimeType The mime type for the data. Empty string translates to application/octet-stream
     * @param _public True if the blob should be publicly available First published in XenServer 6.1.
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task createNewBlobAsync(Connection c, String name, String mimeType, Boolean _public) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.create_new_blob";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(mimeType), Marshalling.toXMLRPC(_public)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Create a placeholder for a named binary blob of data that is associated with this pool
     * First published in XenServer 5.0.
     *
     * @param c The connection the call is made on
     * @param name The name associated with the blob
     * @param mimeType The mime type for the data. Empty string translates to application/octet-stream
     * @param _public True if the blob should be publicly available First published in XenServer 6.1.
     * @return The reference of the blob, needed for populating its data
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Blob createNewBlob(Connection c, String name, String mimeType, Boolean _public) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.create_new_blob";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(mimeType), Marshalling.toXMLRPC(_public)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBlob(result);
    }

    /**
     * This call enables external authentication on all the hosts of the pool
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param config A list of key-values containing the configuration data
     * @param serviceName The name of the service
     * @param authType The type of authentication (e.g. AD for Active Directory)
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void enableExternalAuth(Connection c, Map<String, String> config, String serviceName, String authType) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.enable_external_auth";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(config), Marshalling.toXMLRPC(serviceName), Marshalling.toXMLRPC(authType)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * This call disables external authentication on all the hosts of the pool
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param config Optional parameters as a list of key-values containing the configuration data
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void disableExternalAuth(Connection c, Map<String, String> config) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.disable_external_auth";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(config)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * This call asynchronously detects if the external authentication configuration in any slave is different from that in the master and raises appropriate alerts
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void detectNonhomogeneousExternalAuth(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.detect_nonhomogeneous_external_auth";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Initializes workload balancing monitoring on this pool with the specified wlb server
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param wlbUrl The ip address and port to use when accessing the wlb server
     * @param wlbUsername The username used to authenticate with the wlb server
     * @param wlbPassword The password used to authenticate with the wlb server
     * @param xenserverUsername The username used by the wlb server to authenticate with the xenserver
     * @param xenserverPassword The password used by the wlb server to authenticate with the xenserver
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task initializeWlbAsync(Connection c, String wlbUrl, String wlbUsername, String wlbPassword, String xenserverUsername, String xenserverPassword) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.initialize_wlb";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(wlbUrl), Marshalling.toXMLRPC(wlbUsername), Marshalling.toXMLRPC(wlbPassword), Marshalling.toXMLRPC(xenserverUsername), Marshalling.toXMLRPC(xenserverPassword)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Initializes workload balancing monitoring on this pool with the specified wlb server
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param wlbUrl The ip address and port to use when accessing the wlb server
     * @param wlbUsername The username used to authenticate with the wlb server
     * @param wlbPassword The password used to authenticate with the wlb server
     * @param xenserverUsername The username used by the wlb server to authenticate with the xenserver
     * @param xenserverPassword The password used by the wlb server to authenticate with the xenserver
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void initializeWlb(Connection c, String wlbUrl, String wlbUsername, String wlbPassword, String xenserverUsername, String xenserverPassword) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.initialize_wlb";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(wlbUrl), Marshalling.toXMLRPC(wlbUsername), Marshalling.toXMLRPC(wlbPassword), Marshalling.toXMLRPC(xenserverUsername), Marshalling.toXMLRPC(xenserverPassword)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Permanently deconfigures workload balancing monitoring on this pool
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task deconfigureWlbAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.deconfigure_wlb";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Permanently deconfigures workload balancing monitoring on this pool
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void deconfigureWlb(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.deconfigure_wlb";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Sets the pool optimization criteria for the workload balancing server
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param config The configuration to use in optimizing this pool
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task sendWlbConfigurationAsync(Connection c, Map<String, String> config) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.send_wlb_configuration";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(config)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Sets the pool optimization criteria for the workload balancing server
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param config The configuration to use in optimizing this pool
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void sendWlbConfiguration(Connection c, Map<String, String> config) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.send_wlb_configuration";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(config)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Retrieves the pool optimization criteria from the workload balancing server
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task retrieveWlbConfigurationAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.retrieve_wlb_configuration";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Retrieves the pool optimization criteria from the workload balancing server
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return The configuration used in optimizing this pool
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Map<String, String> retrieveWlbConfiguration(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.retrieve_wlb_configuration";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Retrieves vm migrate recommendations for the pool from the workload balancing server
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task retrieveWlbRecommendationsAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.retrieve_wlb_recommendations";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Retrieves vm migrate recommendations for the pool from the workload balancing server
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return The list of vm migration recommendations
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Map<VM, Set<String>> retrieveWlbRecommendations(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.retrieve_wlb_recommendations";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfVMSetOfString(result);
    }

    /**
     * Send the given body to the given host and port, using HTTPS, and print the response.  This is used for debugging the SSL layer.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param host No description
     * @param port No description
     * @param body No description
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task sendTestPostAsync(Connection c, String host, Long port, String body) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.send_test_post";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(host), Marshalling.toXMLRPC(port), Marshalling.toXMLRPC(body)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Send the given body to the given host and port, using HTTPS, and print the response.  This is used for debugging the SSL layer.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param host No description
     * @param port No description
     * @param body No description
     * @return The response
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static String sendTestPost(Connection c, String host, Long port, String body) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.send_test_post";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(host), Marshalling.toXMLRPC(port), Marshalling.toXMLRPC(body)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Install a TLS CA certificate, pool-wide.
     * First published in XenServer 5.5.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param name A name to give the certificate
     * @param cert The certificate in PEM format
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public static Task certificateInstallAsync(Connection c, String name, String cert) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.certificate_install";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(cert)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Install a TLS CA certificate, pool-wide.
     * First published in XenServer 5.5.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param name A name to give the certificate
     * @param cert The certificate in PEM format
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public static void certificateInstall(Connection c, String name, String cert) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.certificate_install";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(cert)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove a pool-wide TLS CA certificate.
     * First published in XenServer 5.5.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param name The certificate name
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public static Task certificateUninstallAsync(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.certificate_uninstall";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Remove a pool-wide TLS CA certificate.
     * First published in XenServer 5.5.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param name The certificate name
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public static void certificateUninstall(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.certificate_uninstall";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * List the names of all installed TLS CA certificates.
     * First published in XenServer 5.5.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public static Task certificateListAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.certificate_list";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * List the names of all installed TLS CA certificates.
     * First published in XenServer 5.5.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @return All installed certificates
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public static Set<String> certificateList(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.certificate_list";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfString(result);
    }

    /**
     * Install a TLS CA certificate, pool-wide.
     * First published in 1.290.0.
     *
     * @param c The connection the call is made on
     * @param name A name to give the certificate
     * @param cert The certificate in PEM format
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task installCaCertificateAsync(Connection c, String name, String cert) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.install_ca_certificate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(cert)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Install a TLS CA certificate, pool-wide.
     * First published in 1.290.0.
     *
     * @param c The connection the call is made on
     * @param name A name to give the certificate
     * @param cert The certificate in PEM format
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void installCaCertificate(Connection c, String name, String cert) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.install_ca_certificate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(cert)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove a pool-wide TLS CA certificate.
     * First published in 1.290.0.
     *
     * @param c The connection the call is made on
     * @param name The certificate name
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task uninstallCaCertificateAsync(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.uninstall_ca_certificate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Remove a pool-wide TLS CA certificate.
     * First published in 1.290.0.
     *
     * @param c The connection the call is made on
     * @param name The certificate name
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void uninstallCaCertificate(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.uninstall_ca_certificate";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Install a TLS CA-issued Certificate Revocation List, pool-wide.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param name A name to give the CRL
     * @param cert The CRL
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task crlInstallAsync(Connection c, String name, String cert) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.crl_install";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(cert)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Install a TLS CA-issued Certificate Revocation List, pool-wide.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param name A name to give the CRL
     * @param cert The CRL
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void crlInstall(Connection c, String name, String cert) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.crl_install";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name), Marshalling.toXMLRPC(cert)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove a pool-wide TLS CA-issued Certificate Revocation List.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param name The CRL name
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task crlUninstallAsync(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.crl_uninstall";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Remove a pool-wide TLS CA-issued Certificate Revocation List.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @param name The CRL name
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void crlUninstall(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.crl_uninstall";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * List the names of all installed TLS CA-issued Certificate Revocation Lists.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task crlListAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.crl_list";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * List the names of all installed TLS CA-issued Certificate Revocation Lists.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return The names of all installed CRLs
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Set<String> crlList(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.crl_list";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfString(result);
    }

    /**
     * Copy the TLS CA certificates and CRLs of the master to all slaves.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task certificateSyncAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.certificate_sync";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Copy the TLS CA certificates and CRLs of the master to all slaves.
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void certificateSync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.certificate_sync";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Enable TLS server certificate verification
     * First published in 1.290.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void enableTlsVerification(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.enable_tls_verification";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Enable the redo log on the given SR and start using it, unless HA is enabled.
     * First published in XenServer 5.6.
     *
     * @param c The connection the call is made on
     * @param sr SR to hold the redo log.
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task enableRedoLogAsync(Connection c, SR sr) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.enable_redo_log";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(sr)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Enable the redo log on the given SR and start using it, unless HA is enabled.
     * First published in XenServer 5.6.
     *
     * @param c The connection the call is made on
     * @param sr SR to hold the redo log.
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void enableRedoLog(Connection c, SR sr) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.enable_redo_log";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(sr)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Disable the redo log if in use, unless HA is enabled.
     * First published in XenServer 5.6.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Task disableRedoLogAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.disable_redo_log";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Disable the redo log if in use, unless HA is enabled.
     * First published in XenServer 5.6.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static void disableRedoLog(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.disable_redo_log";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the IP address of the vswitch controller.
     * First published in XenServer 5.6.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param address IP address of the vswitch controller.
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public static Task setVswitchControllerAsync(Connection c, String address) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_vswitch_controller";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(address)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Set the IP address of the vswitch controller.
     * First published in XenServer 5.6.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param address IP address of the vswitch controller.
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public static void setVswitchController(Connection c, String address) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_vswitch_controller";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(address)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * This call tests if a location is valid
     * First published in XenServer 5.6 FP1.
     *
     * @param c The connection the call is made on
     * @param config Location config settings to test
     * @return An XMLRPC result
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String testArchiveTarget(Connection c, Map<String, String> config) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.test_archive_target";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(config)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * This call attempts to enable pool-wide local storage caching
     * First published in XenServer 5.6 FP1.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task enableLocalStorageCachingAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.enable_local_storage_caching";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * This call attempts to enable pool-wide local storage caching
     * First published in XenServer 5.6 FP1.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void enableLocalStorageCaching(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.enable_local_storage_caching";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * This call disables pool-wide local storage caching
     * First published in XenServer 5.6 FP1.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task disableLocalStorageCachingAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.disable_local_storage_caching";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * This call disables pool-wide local storage caching
     * First published in XenServer 5.6 FP1.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void disableLocalStorageCaching(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.disable_local_storage_caching";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * This call returns the license state for the pool
     * First published in XenServer 6.2.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task getLicenseStateAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.get_license_state";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * This call returns the license state for the pool
     * First published in XenServer 6.2.
     *
     * @param c The connection the call is made on
     * @return The pool's license state
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Map<String, String> getLicenseState(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_license_state";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfStringString(result);
    }

    /**
     * Apply an edition to all hosts in the pool
     * First published in XenServer 6.2.
     *
     * @param c The connection the call is made on
     * @param edition The requested edition
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task applyEditionAsync(Connection c, String edition) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.apply_edition";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(edition)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Apply an edition to all hosts in the pool
     * First published in XenServer 6.2.
     *
     * @param c The connection the call is made on
     * @param edition The requested edition
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void applyEdition(Connection c, String edition) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.apply_edition";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(edition)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Sets ssl_legacy true on each host, pool-master last. See Host.ssl_legacy and Host.set_ssl_legacy.
     * First published in XenServer 7.0.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public Task enableSslLegacyAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.enable_ssl_legacy";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Sets ssl_legacy true on each host, pool-master last. See Host.ssl_legacy and Host.set_ssl_legacy.
     * First published in XenServer 7.0.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public void enableSslLegacy(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.enable_ssl_legacy";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Sets ssl_legacy false on each host, pool-master last. See Host.ssl_legacy and Host.set_ssl_legacy.
     * First published in XenServer 7.0.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public Task disableSslLegacyAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.disable_ssl_legacy";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Sets ssl_legacy false on each host, pool-master last. See Host.ssl_legacy and Host.set_ssl_legacy.
     * First published in XenServer 7.0.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public void disableSslLegacy(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.disable_ssl_legacy";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Enable or disable IGMP Snooping on the pool.
     * First published in XenServer 7.3.
     *
     * @param c The connection the call is made on
     * @param value Enable or disable IGMP Snooping on the pool
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task setIgmpSnoopingEnabledAsync(Connection c, Boolean value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_igmp_snooping_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Enable or disable IGMP Snooping on the pool.
     * First published in XenServer 7.3.
     *
     * @param c The connection the call is made on
     * @param value Enable or disable IGMP Snooping on the pool
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setIgmpSnoopingEnabled(Connection c, Boolean value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_igmp_snooping_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Return true if the extension is available on the pool
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param name The name of the API call
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task hasExtensionAsync(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.has_extension";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Return true if the extension is available on the pool
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param name The name of the API call
     * @return True if the extension exists, false otherwise
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Boolean hasExtension(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.has_extension";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toBoolean(result);
    }

    /**
     * Add a key-value pair to the pool-wide guest agent configuration
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param key The key to add
     * @param value The value to add
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task addToGuestAgentConfigAsync(Connection c, String key, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.add_to_guest_agent_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Add a key-value pair to the pool-wide guest agent configuration
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param key The key to add
     * @param value The value to add
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void addToGuestAgentConfig(Connection c, String key, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.add_to_guest_agent_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove a key-value pair from the pool-wide guest agent configuration
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param key The key to remove
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task removeFromGuestAgentConfigAsync(Connection c, String key) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.remove_from_guest_agent_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Remove a key-value pair from the pool-wide guest agent configuration
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param key The key to remove
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void removeFromGuestAgentConfig(Connection c, String key) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.remove_from_guest_agent_config";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(key)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * 
     * First published in Citrix Hypervisor 8.2 Hotfix 2.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.InternalError The server failed to handle your request, due to an internal error. The given message may give details useful for debugging the problem.
     * @throws Types.HostIsSlave You cannot make regular API calls directly on a supporter. Please pass API calls via the coordinator host.
     * @throws Types.CannotContactHost Cannot forward messages because the server cannot be contacted. The server may be switched off or there may be network connectivity problems.
     * @throws Types.HaIsEnabled The operation could not be performed because HA is enabled on the Pool
     * @throws Types.NotSupportedDuringUpgrade This operation is not supported during an upgrade.
     */
    public static Task rotateSecretAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.InternalError,
       Types.HostIsSlave,
       Types.CannotContactHost,
       Types.HaIsEnabled,
       Types.NotSupportedDuringUpgrade {
        String method_call = "Async.pool.rotate_secret";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * 
     * First published in Citrix Hypervisor 8.2 Hotfix 2.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     * @throws Types.InternalError The server failed to handle your request, due to an internal error. The given message may give details useful for debugging the problem.
     * @throws Types.HostIsSlave You cannot make regular API calls directly on a supporter. Please pass API calls via the coordinator host.
     * @throws Types.CannotContactHost Cannot forward messages because the server cannot be contacted. The server may be switched off or there may be network connectivity problems.
     * @throws Types.HaIsEnabled The operation could not be performed because HA is enabled on the Pool
     * @throws Types.NotSupportedDuringUpgrade This operation is not supported during an upgrade.
     */
    public static void rotateSecret(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException,
       Types.InternalError,
       Types.HostIsSlave,
       Types.CannotContactHost,
       Types.HaIsEnabled,
       Types.NotSupportedDuringUpgrade {
        String method_call = "pool.rotate_secret";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set enabled set of repositories
     * First published in 1.301.0.
     *
     * @param c The connection the call is made on
     * @param value The set of repositories to be enabled
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task setRepositoriesAsync(Connection c, Set<Repository> value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_repositories";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Set enabled set of repositories
     * First published in 1.301.0.
     *
     * @param c The connection the call is made on
     * @param value The set of repositories to be enabled
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setRepositories(Connection c, Set<Repository> value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_repositories";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Add a repository to the enabled set
     * First published in 1.301.0.
     *
     * @param c The connection the call is made on
     * @param value The repository to be added to the enabled set
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task addRepositoryAsync(Connection c, Repository value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.add_repository";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Add a repository to the enabled set
     * First published in 1.301.0.
     *
     * @param c The connection the call is made on
     * @param value The repository to be added to the enabled set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void addRepository(Connection c, Repository value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.add_repository";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Remove a repository from the enabled set
     * First published in 1.301.0.
     *
     * @param c The connection the call is made on
     * @param value The repository to be removed
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task removeRepositoryAsync(Connection c, Repository value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.remove_repository";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Remove a repository from the enabled set
     * First published in 1.301.0.
     *
     * @param c The connection the call is made on
     * @param value The repository to be removed
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void removeRepository(Connection c, Repository value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.remove_repository";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Sync with the enabled repository
     * First published in 1.329.0.
     *
     * @param c The connection the call is made on
     * @param force If true local mirroring repo will be removed before syncing
     * @param token The token for repository client authentication
     * @param tokenId The ID of the token
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task syncUpdatesAsync(Connection c, Boolean force, String token, String tokenId) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.sync_updates";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(force), Marshalling.toXMLRPC(token), Marshalling.toXMLRPC(tokenId)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Sync with the enabled repository
     * First published in 1.329.0.
     *
     * @param c The connection the call is made on
     * @param force If true local mirroring repo will be removed before syncing
     * @param token The token for repository client authentication
     * @param tokenId The ID of the token
     * @return The SHA256 hash of updateinfo.xml.gz
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public String syncUpdates(Connection c, Boolean force, String token, String tokenId) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.sync_updates";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(force), Marshalling.toXMLRPC(token), Marshalling.toXMLRPC(tokenId)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toString(result);
    }

    /**
     * Check if the pool is ready to be updated. If not, report the reasons.
     * First published in 1.304.0.
     *
     * @param c The connection the call is made on
     * @param requiresReboot Assume that the update will require host reboots
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task checkUpdateReadinessAsync(Connection c, Boolean requiresReboot) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.check_update_readiness";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(requiresReboot)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Check if the pool is ready to be updated. If not, report the reasons.
     * First published in 1.304.0.
     *
     * @param c The connection the call is made on
     * @param requiresReboot Assume that the update will require host reboots
     * @return A set of error codes with arguments, if the pool is
        not ready to update. An empty list means the pool can be updated.
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Set<Set<String>> checkUpdateReadiness(Connection c, Boolean requiresReboot) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.check_update_readiness";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(requiresReboot)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfSetOfString(result);
    }

    /**
     * Enable client certificate authentication on the pool
     * First published in 1.318.0.
     *
     * @param c The connection the call is made on
     * @param name The name (CN/SAN) that an incoming client certificate must have to allow authentication
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task enableClientCertificateAuthAsync(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.enable_client_certificate_auth";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Enable client certificate authentication on the pool
     * First published in 1.318.0.
     *
     * @param c The connection the call is made on
     * @param name The name (CN/SAN) that an incoming client certificate must have to allow authentication
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void enableClientCertificateAuth(Connection c, String name) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.enable_client_certificate_auth";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(name)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Disable client certificate authentication on the pool
     * First published in 1.318.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task disableClientCertificateAuthAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.disable_client_certificate_auth";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Disable client certificate authentication on the pool
     * First published in 1.318.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void disableClientCertificateAuth(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.disable_client_certificate_auth";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Configure proxy for RPM package repositories.
     * First published in 21.3.0.
     *
     * @param c The connection the call is made on
     * @param url The URL of the proxy server
     * @param username The username used to authenticate with the proxy server
     * @param password The password used to authenticate with the proxy server
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task configureRepositoryProxyAsync(Connection c, String url, String username, String password) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.configure_repository_proxy";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(url), Marshalling.toXMLRPC(username), Marshalling.toXMLRPC(password)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Configure proxy for RPM package repositories.
     * First published in 21.3.0.
     *
     * @param c The connection the call is made on
     * @param url The URL of the proxy server
     * @param username The username used to authenticate with the proxy server
     * @param password The password used to authenticate with the proxy server
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void configureRepositoryProxy(Connection c, String url, String username, String password) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.configure_repository_proxy";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(url), Marshalling.toXMLRPC(username), Marshalling.toXMLRPC(password)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Disable the proxy for RPM package repositories.
     * First published in 21.4.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task disableRepositoryProxyAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.disable_repository_proxy";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Disable the proxy for RPM package repositories.
     * First published in 21.4.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void disableRepositoryProxy(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.disable_repository_proxy";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the UEFI certificates for a pool and all its hosts. Deprecated: use set_custom_uefi_certificates instead
     * First published in 22.16.0.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param value The certificates to apply to the pool and its hosts
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public Task setUefiCertificatesAsync(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_uefi_certificates";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Set the UEFI certificates for a pool and all its hosts. Deprecated: use set_custom_uefi_certificates instead
     * First published in 22.16.0.
     * @deprecated
     *
     * @param c The connection the call is made on
     * @param value The certificates to apply to the pool and its hosts
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
   @Deprecated public void setUefiCertificates(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_uefi_certificates";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set custom UEFI certificates for a pool and all its hosts. Need `allow-custom-uefi-verts` set to true in conf. If empty: default back to Pool.uefi_certificates
     * Experimental. First published in 23.25.0-next.
     *
     * @param c The connection the call is made on
     * @param value The certificates to apply to the pool and its hosts
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task setCustomUefiCertificatesAsync(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_custom_uefi_certificates";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Set custom UEFI certificates for a pool and all its hosts. Need `allow-custom-uefi-verts` set to true in conf. If empty: default back to Pool.uefi_certificates
     * Experimental. First published in 23.25.0-next.
     *
     * @param c The connection the call is made on
     * @param value The certificates to apply to the pool and its hosts
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setCustomUefiCertificates(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_custom_uefi_certificates";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * updates all the host firewalls in the pool to open or close port 80 depending on the value
     * Experimental. First published in 22.27.0.
     *
     * @param c The connection the call is made on
     * @param value true - http port 80 will be blocked, false - http port 80 will be open for the hosts in the pool
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task setHttpsOnlyAsync(Connection c, Boolean value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_https_only";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * updates all the host firewalls in the pool to open or close port 80 depending on the value
     * Experimental. First published in 22.27.0.
     *
     * @param c The connection the call is made on
     * @param value true - http port 80 will be blocked, false - http port 80 will be open for the hosts in the pool
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setHttpsOnly(Connection c, Boolean value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_https_only";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Set the timestamp for the next telemetry data collection.
     * Experimental. First published in 23.9.0.
     *
     * @param c The connection the call is made on
     * @param value The earliest timestamp (in UTC) when the next round of telemetry collection can be carried out.
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task setTelemetryNextCollectionAsync(Connection c, Date value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_telemetry_next_collection";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Set the timestamp for the next telemetry data collection.
     * Experimental. First published in 23.9.0.
     *
     * @param c The connection the call is made on
     * @param value The earliest timestamp (in UTC) when the next round of telemetry collection can be carried out.
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setTelemetryNextCollection(Connection c, Date value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_telemetry_next_collection";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Assign a new UUID to telemetry data.
     * Experimental. First published in 23.9.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task resetTelemetryUuidAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.reset_telemetry_uuid";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Assign a new UUID to telemetry data.
     * Experimental. First published in 23.9.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void resetTelemetryUuid(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.reset_telemetry_uuid";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Configure periodic update synchronization to sync updates from a remote CDN
     * Experimental. First published in 23.18.0.
     *
     * @param c The connection the call is made on
     * @param updateSyncFrequency The frequency at which updates are synchronized from a remote CDN: daily or weekly.
     * @param updateSyncDay The day of the week the update synchronization will happen, based on pool's local timezone. Valid values are 0 to 6, 0 being Sunday. For 'daily' schedule, the value is ignored.
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task configureUpdateSyncAsync(Connection c, Types.UpdateSyncFrequency updateSyncFrequency, Long updateSyncDay) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.configure_update_sync";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(updateSyncFrequency), Marshalling.toXMLRPC(updateSyncDay)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * Configure periodic update synchronization to sync updates from a remote CDN
     * Experimental. First published in 23.18.0.
     *
     * @param c The connection the call is made on
     * @param updateSyncFrequency The frequency at which updates are synchronized from a remote CDN: daily or weekly.
     * @param updateSyncDay The day of the week the update synchronization will happen, based on pool's local timezone. Valid values are 0 to 6, 0 being Sunday. For 'daily' schedule, the value is ignored.
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void configureUpdateSync(Connection c, Types.UpdateSyncFrequency updateSyncFrequency, Long updateSyncDay) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.configure_update_sync";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(updateSyncFrequency), Marshalling.toXMLRPC(updateSyncDay)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * enable or disable periodic update synchronization depending on the value
     * Experimental. First published in 23.18.0.
     *
     * @param c The connection the call is made on
     * @param value true - enable periodic update synchronization, false - disable it
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public Task setUpdateSyncEnabledAsync(Connection c, Boolean value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "Async.pool.set_update_sync_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toTask(result);
    }

    /**
     * enable or disable periodic update synchronization depending on the value
     * Experimental. First published in 23.18.0.
     *
     * @param c The connection the call is made on
     * @param value true - enable periodic update synchronization, false - disable it
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public void setUpdateSyncEnabled(Connection c, Boolean value) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.set_update_sync_enabled";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session), Marshalling.toXMLRPC(this.ref), Marshalling.toXMLRPC(value)};
        Map response = c.dispatch(method_call, method_params);
    }

    /**
     * Return a list of all the pools known to the system.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return references to all objects
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Set<Pool> getAll(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_all";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toSetOfPool(result);
    }

    /**
     * Return a map of pool references to pool records for all pools known to the system.
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return records of all objects
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException Thrown if the call failed.
     * @throws XmlRpcException Thrown if the result of an asynchronous call could not be parsed.
     */
    public static Map<Pool, Pool.Record> getAllRecords(Connection c) throws
       BadServerResponse,
       XenAPIException,
       XmlRpcException {
        String method_call = "pool.get_all_records";
        String session = c.getSessionReference();
        Object[] method_params = {Marshalling.toXMLRPC(session)};
        Map response = c.dispatch(method_call, method_params);
        Object result = response.get("Value");
        return Types.toMapOfPoolPoolRecord(result);
    }

}